==> dvdrtools_0.2.1.orig.tar.gz: dvdrtools-0.2.1/libschily/stdio/flag.c <==
/* @(#)flag.c	2.7 00/12/03 Copyright 1986 J. Schilling */
/*
 *	Copyright (c) 1986 J. Schilling
 */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */

#include "io.h"
#include <stdxlib.h>

#ifdef	DO_MYFLAG

#define	FL_INIT	10

int	_io_glflag;		/* global default flag */
int	_fl_inc = 10;		/* increment for expanding flag struct */
int	_fl_max = FL_INIT;	/* max fd currently in _io_myfl */
_io_fl	_io_smyfl[FL_INIT];	/* initial static space */
_io_fl	*_io_myfl = _io_smyfl;	/* init to static space */

LOCAL int _more_flags	__PR((FILE * ));

LOCAL int
_more_flags(FILE *fp)
{
	register int	f = fileno(fp);
	register int	n = _fl_max;
	register _io_fl	*np;

	while (n <= f)
		n += _fl_inc;

	if (_io_myfl == _io_smyfl) {
		np = (_io_fl *) malloc(n * sizeof(*np));
		fillbytes(np, n * sizeof(*np), '\0');
		movebytes(_io_smyfl, np, sizeof(_io_smyfl)/sizeof(*np));
	} else {
		np = (_io_fl *) realloc(_io_myfl, n * sizeof(*np));
		if (np)
			fillbytes(&np[_fl_max], (n-_fl_max)*sizeof(*np), '\0');
	}
	if (np) {
		_io_myfl = np;
		_fl_max = n;
		return (_io_get_my_flag(fp));
	} else {
		return (_IONORAISE);
	}
}

EXPORT int
_io_get_my_flag(FILE *fp)
{
	register int	f = fileno(fp);
	register _io_fl	*fl;

	if (f >= _fl_max)
		return (_more_flags(fp));

	fl = &_io_myfl[f];

	if (fl->fl_io == 0 || fl->fl_io == fp)
		return (fl->fl_flags);

	while (fl && fl->fl_io != fp)
		fl = fl->fl_next;

	if (fl == 0)
		return (0);

	return (fl->fl_flags);
}

EXPORT void
_io_set_my_flag(FILE *fp, int flag)
{
	register int	f = fileno(fp);
	register _io_fl	*fl;
	register _io_fl	*fl2;

	if (f >= _fl_max)
		(void) _more_flags(fp);

	fl = &_io_myfl[f];

	if (fl->fl_io != (FILE *)0) {
		fl2 = fl;

		while (fl && fl->fl_io != fp)
			fl = fl->fl_next;
		if (fl == 0) {
			if ((fl = (_io_fl *) malloc(sizeof(*fl))) == 0)
				return;
			fl->fl_next = fl2->fl_next;
			fl2->fl_next = fl;
		}
	}
	fl->fl_io = fp;
	fl->fl_flags = flag;
}

EXPORT void
_io_add_my_flag(FILE *fp, int flag)
{
	int	oflag = _io_get_my_flag(fp);

	oflag |= flag;

	_io_set_my_flag(fp, oflag);
}

#endif	/* DO_MYFLAG */

==> dvdrtools_0.3.1.orig.tar.gz: dvdrtools-0.3.1/libschily/stdio/flag.c <==
/* @(#)flag.c	2.7 00/12/03 Copyright 1986 J. Schilling */
/*
 *	Copyright (c) 1986 J. Schilling
 */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */

#include "io.h"
#include <stdxlib.h>
#include <string.h>

#ifdef	DO_MYFLAG

#define	FL_INIT	10

int	_io_glflag;		/* global default flag */
int	_fl_inc = 10;		/* increment for expanding flag struct */
int	_fl_max = FL_INIT;	/* max fd currently in _io_myfl */
_io_fl	_io_smyfl[FL_INIT];	/* initial static space */
_io_fl	*_io_myfl = _io_smyfl;	/* init to static space */

LOCAL int _more_flags	__PR((FILE * ));

LOCAL int
_more_flags(FILE *fp)
{
	register int	f = fileno(fp);
	register int	n = _fl_max;
	register _io_fl	*np;

	while (n <= f)
		n += _fl_inc;

	if (_io_myfl == _io_smyfl) {
		int npsize;
		np = (_io_fl *) malloc(n * sizeof(*np));
		npsize = sizeof(_io_smyfl)/sizeof(*np);
		memset(np+npsize, 0, n * sizeof(*np)-npsize);
		memmove(np, _io_smyfl, npsize);
	} else {
		np = (_io_fl *) realloc(_io_myfl, n * sizeof(*np));
		if (np)
			memset(&np[_fl_max], 0, (n-_fl_max)*sizeof(*np));
	}
	if (np) {
		_io_myfl = np;
		_fl_max = n;
		return (_io_get_my_flag(fp));
	} else {
		return (_IONORAISE);
	}
}

EXPORT int
_io_get_my_flag(FILE *fp)
{
	register int	f = fileno(fp);
	register _io_fl	*fl;

	if (f >= _fl_max)
		return (_more_flags(fp));

	fl = &_io_myfl[f];

	if (fl->fl_io == 0 || fl->fl_io == fp)
		return (fl->fl_flags);

	while (fl && fl->fl_io != fp)
		fl = fl->fl_next;

	if (fl == 0)
		return (0);

	return (fl->fl_flags);
}

EXPORT void
_io_set_my_flag(FILE *fp, int flag)
{
	register int	f = fileno(fp);
	register _io_fl	*fl;
	register _io_fl	*fl2;

	if (f >= _fl_max)
		(void) _more_flags(fp);

	fl = &_io_myfl[f];

	if (fl->fl_io != (FILE *)0) {
		fl2 = fl;

		while (fl && fl->fl_io != fp)
			fl = fl->fl_next;
		if (fl == 0) {
			if ((fl = (_io_fl *) malloc(sizeof(*fl))) == 0)
				return;
			fl->fl_next = fl2->fl_next;
			fl2->fl_next = fl;
		}
	}
	fl->fl_io = fp;
	fl->fl_flags = flag;
}

EXPORT void
_io_add_my_flag(FILE *fp, int flag)
{
	int	oflag = _io_get_my_flag(fp);

	oflag |= flag;

	_io_set_my_flag(fp, oflag);
}

#endif	/* DO_MYFLAG */
